# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - SendMessage Action

    @copyright: 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.action import ActionBase
from MoinMoin.log import getLogger
from MoinMessage import GPG, MoinMessageError, Message, sendMessage
from MoinSupport import *
from MoinMoin.wikiutil import escape
from email.mime.text import MIMEText

Dependencies = []

class SendMessage(ActionBase, ActionSupport):

    "An action that can send a message to another site."

    def get_form_html(self, buttons_html):

        "Present an interface for message sending."

        _ = self._
        request = self.request
        form = self.get_form()

        message = form.get("message", [""])[0]
        recipient = form.get("recipient", [""])[0]

        # Get a list of potential recipients.

        recipients = self.get_recipients()

        # Prepare the recipients list, selecting the specified recipients.

        recipients_list = []

        if recipients:
            recipients_list += self.get_option_list(recipient, recipients) or []

        recipients_list.sort()

        # Fill in the fields and labels.

        d = {
            "buttons_html"          : buttons_html,
            "recipient_label"       : _("Recipient"),
            "recipients_list"       : "\n".join(recipients_list),
            "message_label"         : _("Message text"),
            "message_default"       : escattr(message),
            }

        # Prepare the output HTML.

        html = '''
<table>
    <tr>
        <td class="label"><label>%(recipient_label)s</label></td>
        <td>
            <select name="recipient">
                %(recipients_list)s
            </select>
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(message_label)s</label></td>
        <td colspan="2">
            <input name="message" type="text" size="40" value="%(message_default)s" />
        </td>
    </tr>
    <tr>
        <td></td>
        <td colspan="2" class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>''' % d

        return html

    def do_action(self):

        "Attempt to send the message."

        _ = self._
        request = self.request
        form = self.get_form()

        text = form.get("message", [None])[0]
        recipient = form.get("recipient", [None])[0]

        if not text:
            return 0, _("A message must be given.")

        if not recipient:
            return 0, _("A recipient must be given.")

        homedir = self.get_homedir()
        if not homedir:
            return 0, _("MoinMessage has not been set up: a GPG homedir is not defined.")

        gpg = GPG(homedir)

        # Construct a message from the request.

        message = Message()
        message.add_update(MIMEText(text, "moin"))

        # Get the sender details for signing messages.
        # This is not the same as the details for authenticating users in the
        # PostMessage action since the fingerprints refer to public keys.

        signing_users = getWikiDict(getattr(request.cfg, "moinmessage_gpg_signing_users_page", "MoinMessageSigningUserDict"), request)
        signer = signing_users and signing_users.get(request.user.name)

        # Get the recipient details.

        recipients = self.get_recipients()
        if not recipients:
            return 0, _("No recipients page is defined for MoinMessage.")

        recipient_details = recipients.get(recipient)
        if not recipient_details:
            return 0, _("The specified recipient is not present in the list of known contacts.")

        try:
            url, fingerprint = recipient_details.split()
        except ValueError:
            return 0, _("The recipient details were not in the correct format: url, fingerprint.")

        # Sign, encrypt and send the message.

        try:
            message = message.get_payload()
            if signer:
                message = gpg.signMessage(message, signer)

            message = gpg.encryptMessage(message, fingerprint)
            sendMessage(message, url)

        except MoinMessageError, exc:
            return 0, "%s: %s" % (_("The message could not be prepared and sent:"), exc)

        return 1, None

    def get_homedir(self):

        "Locate the GPG home directory."

        return getattr(self.request.cfg, "moinmessage_gpg_homedir")

    def get_recipients(self):
        return getWikiDict(getattr(self.request.cfg, "moinmessage_gpg_recipients_page", "MoinMessageRecipientsDict"), self.request)

# Action function.

def execute(pagename, request):
    SendMessage(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
